package network

import (
	"context"
	"crypto/ed25519"
	"log/slog"
	"net"

	"koti.casa/numenor-labs/dsfx/pkg/crypto/identity"
	"koti.casa/numenor-labs/dsfx/pkg/handshake"
)

// Listener ...
type Listener struct {
	logger      *slog.Logger
	tcpListener *net.TCPListener
	identity    ed25519.PrivateKey
}

// Accept implements net.Listener.
func (l *Listener) Accept() (net.Conn, error) {
	ctx := context.Background()

	conn, err := l.tcpListener.AcceptTCP()
	if err != nil {
		return nil, err
	}

	clientIdentity, sessionKey, err := handshake.Accept(ctx, conn, l.identity)
	if err != nil {
		return nil, err
	}

	return NewConn(conn, sessionKey, identity.ToPublicKey(l.identity), clientIdentity), nil
}

// Close implements net.Listener.
func (l *Listener) Close() error {
	return l.tcpListener.Close()
}

// Addr implements net.Listener.
func (l *Listener) Addr() net.Addr {
	laddr := l.tcpListener.Addr().(*net.TCPAddr)

	return NewAddr(laddr.IP, laddr.Port, identity.ToPublicKey(l.identity))
}
